@@ -21,8 +21,8 @@ include/global.h - Shared definitions for user-space apps
 
   -- For details see doc/GPL2.txt.
 =============================================================================*/
-#ifndef RPL_GLOBAL_H
-#define RPL_GLOBAL_H 1
+#ifndef TTYRPLD_GLOBAL_H
+#define TTYRPLD_GLOBAL_H 1
 
 #include <endian.h>
 #include <stdint.h>
@@ -30,26 +30,24 @@ include/global.h - Shared definitions for user-space apps
 #ifndef O_BINARY
 #    define O_BINARY 0
 #endif
-#if __BYTE_ORER == __BIG_ENDIAN
-#    define BIGENDIAN 1
-#    define PP_swab(ptr, sz) G_swab((ptr), (sz))
+#if __BYTE_ORDER == __BIG_ENDIAN
+#    define PP_swab(ptr, sz) swab_be((ptr), (sz))
 #else
 #    define PP_swab(ptr, sz)
 #endif
 
-#define MAXFNLEN 1024
-#define TTYRPLD_VERSION "0.88"
+#define MAXFNLEN 256
+#define TTYRPLD_VERSION "1.00"
 
-extern int K_MINORBITS(void);
 extern int count_args(const char **);
 extern ssize_t read_wait(int, void *, size_t);
 extern off_t G_skip(int, off_t, int);
-extern void G_swab(void *, size_t);
+extern void swab_be(void *, size_t);
 
 inline static unsigned int min_uint(unsigned int a, unsigned int b) {
     return a < b ? a : b;
 }
 
-#endif // RPL_GLOBAL_H
+#endif // TTYRPLD_GLOBAL_H
 
 //==[ End of file ]============================================================
