@@ -21,8 +21,8 @@ include/rdsh.h - Shared definitions for RPLD/INFO/RPLINFO
 
   -- For details see doc/GPL2.txt.
 =============================================================================*/
-#ifndef RPL_RDSH_H
-#define RPL_RDSH_H 1
+#ifndef TTYRPLD_RDSH_H
+#define TTYRPLD_RDSH_H 1
 
 #include <sys/types.h>
 #include <pthread.h>
@@ -90,6 +90,6 @@ extern void log_close(struct tty *);
 extern int notify(int, unsigned int, const char *, ...);
 extern ssize_t send_wait(int, const void *, size_t, int);
 
-#endif // RPL_RDSH_H
+#endif // TTYRPLD_RDSH_H
 
 //==[ End of file ]============================================================
