@@ -24,6 +24,7 @@ kernel/rpldev.c - Kernel interface for RPLD
 #include <asm/byteorder.h>
 #include <asm/semaphore.h>
 #include <asm/uaccess.h>
+#include <linux/compiler.h>
 #include <linux/fs.h>
 #include <linux/init.h>
 #include <linux/kernel.h>
@@ -38,10 +39,8 @@ kernel/rpldev.c - Kernel interface for RPLD
 #include "../include/rpl_ioctl.h"
 #include "../include/rpl_packet.h"
 
-#ifdef __LITTLE_ENDIAN
-#    define PP_swab(ptr)
-#else
-#    define PP_swab(ptr) my_swab((ptr), sizeof(*(ptr)))
+#ifndef __user
+#    define __user
 #endif
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0)
@@ -54,9 +53,13 @@ kernel/rpldev.c - Kernel interface for RPLD
 #ifndef KBUILD_BASENAME
 #    define KBUILD_BASENAME "rpldev"
 #endif
-#define __STRINGIFY_EXP(__str) #__str
-#define __STRINGIFY(__str) __STRINGIFY_EXP(__str)
-#define PREFIX __STRINGIFY(KBUILD_BASENAME) ": "
+#define PREFIX __stringify(KBUILD_BASENAME) ": "
+
+#ifdef __LITTLE_ENDIAN
+#    define PP_swab(ptr)
+#else
+#    define PP_swab(ptr) my_swab((ptr), sizeof(*(ptr)))
+#endif
 
 // Stage 2 functions
 static int krn_open(struct tty_struct *, struct tty_struct *);
@@ -68,7 +71,7 @@ static int krn_ioctl(struct tty_struct *, struct tty_struct *, unsigned int,
 
 // Stage 3 functions
 static int uif_open(struct inode *, struct file *);
-static ssize_t uif_read(struct file *, char *, size_t, loff_t *);
+static ssize_t uif_read(struct file *, char __user *, size_t, loff_t *);
 static loff_t uif_llseek(struct file *, loff_t, int);
 static int uif_ioctl(struct inode *, struct file *, unsigned int,
  unsigned long);
@@ -82,7 +85,7 @@ inline static unsigned int imin_uint(unsigned int, unsigned int);
 inline static void get_tty_devnr(uint32_t *, struct tty_struct *);
 static int mv_buffer(struct rpld_packet *, const char *, size_t);
 inline static void mv_buffer2(const void *, size_t);
-inline static void mv_user(void *, size_t);
+inline static void mv_user(char __user *, size_t);
 inline static void my_swab(void *, size_t);
 
 // Variables
@@ -90,7 +93,8 @@ static DECLARE_WAIT_QUEUE_HEAD(Inqueue);
 static DECLARE_MUTEX(Open_lock);
 static DECLARE_MUTEX(Buffer_lock);
 static char *Buffer = NULL, *BufRP = NULL, *BufWP = NULL;
-static size_t Bufsize = 16384; // Each tty has a buffer of 4096
+// Each tty has a buffer of 4096, yet the number of possible ttys is endless
+static size_t Bufsize = 16384;
 static int Minor_nr = MISC_DYNAMIC_MINOR;
 static struct file_operations Fops = {
     open:    uif_open,
@@ -126,7 +130,7 @@ int __init init_module(void) {
         cleanup_module();
         return eax;
     }
-    printk("RPL device registered at minor %d\n", Misc_info.minor);
+    printk(PREFIX "Got registered at minor %d\n", Misc_info.minor);
     return 0;
 }
 
@@ -232,8 +236,6 @@ static int uif_open(struct inode *inode, struct file *filp) {
     rpl_qclose = krn_close;
     rpl_qioctl = krn_ioctl;
 
-    printk(KERN_DEBUG PREFIX "RPL device opened\n");
-
     /* The inode's times are changed as follows:
             Access Time: if data is read from the device
       Inode Change Time: when the device is successfully opened
@@ -243,15 +245,11 @@ static int uif_open(struct inode *inode, struct file *filp) {
     return 0;
 }
 
-static ssize_t uif_read(struct file *filp, char *buf, size_t count,
+static ssize_t uif_read(struct file *filp, char __user *buf, size_t count,
  loff_t *ppos) {
     // Nothing read, nothing done
     if(count == 0) { return 0; }
 
-    /* No seeking allowed. No positional reading allowed either. For skipping
-    data, use SEEK_CUR with lseek(). */
-    if(ppos != &filp->f_pos) { return -ESPIPE; }
-
     // Must sleep as long as there is no data.
     if(down_interruptible(&Buffer_lock)) { return -ERESTARTSYS; }
     while(BufRP == BufWP) {
@@ -311,7 +309,6 @@ static unsigned int uif_poll(struct file *filp, poll_table *wait) {
 }
 
 static int uif_close(struct inode *inode, struct file *filp) {
-    printk(KERN_DEBUG PREFIX "RPL device closed\n");
     rpl_qopen  = NULL;
     rpl_qread  = NULL;
     rpl_qwrite = NULL;
@@ -393,7 +390,7 @@ inline static void mv_buffer2(const void *src, size_t count) {
     return;
 }
 
-inline static void mv_user(void *dest, size_t count) {
+inline static void mv_user(char __user *dest, size_t count) {
     /* mv_user() reads COUNT bytes from the circulary buffer and puts it into
     userspace memory. */
     int x = Buffer + Bufsize - BufRP;
