@@ -287,15 +287,13 @@ static void block_signals(void) {
 }
 
 static void print_entry_text(int fd, struct tty *tty) {
-    char buf[64], status = ' ';
+    char buf[64], status;
 
     switch(tty->status) {
-        case IFP_ACTIVATE:
-            status = '*';
-            break;
-        case IFP_DEACTIVSES:
-            status = '.';
-            break;
+        case IFP_ACTIVATE:   status = '*'; break;
+        case IFP_DEACTIVATE: status = ' '; break;
+        case IFP_DEACTIVSES: status = '.'; break;
+        default:             status = '?'; break;
     }
 
     G_devname_nm(tty->dev, buf, sizeof(buf));
@@ -308,15 +306,10 @@ static void print_entry(int fd, struct tty *tty) {
     char buf[64], status = 'x';
 
     switch(tty->status) {
-        case IFP_ACTIVATE:
-            status = 'A';
-            break;
-        case IFP_DEACTIVATE:
-            status = 'D';
-            break;
-        case IFP_DEACTIVSES:
-            status = 'S';
-            break;
+        case IFP_ACTIVATE:   status = 'A'; break;
+        case IFP_DEACTIVATE: status = 'J'; break;
+        case IFP_DEACTIVSES: status = 'D'; break;
+        default:             status = '?'; break;
     }
 
     G_devname_nm(tty->dev, buf, sizeof(buf));
