@@ -1,6 +1,6 @@
 /*=============================================================================
 ttyrpld - TTY replay daemon
-user/rdsh.c - Shared functions for RPLD/INFOD/RPLINFO
+user/rdsh.c - Shared functions for RPLD/INFOD/RPLCTL
   Copyright (C) Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2004
   -- License restrictions apply (GPL2)
 
@@ -24,10 +24,12 @@ user/rdsh.c - Shared functions for RPLD/INFOD/RPLINFO
 #include <sys/socket.h>
 #include <sys/types.h>
 #include <errno.h>
+#include <pthread.h>
 #include <sched.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <syslog.h>
 
 #include <libHX.h>
@@ -35,7 +37,11 @@ user/rdsh.c - Shared functions for RPLD/INFOD/RPLINFO
 #include "global.h"
 #include "rdsh.h"
 
-// Variables
+// Functions
+inline static int K_MINORBITS(void);
+static void setup_kminorbits(void);
+
+// External Variables
 pthread_mutex_t Ttys_lock = PTHREAD_MUTEX_INITIALIZER;
 struct HXbtree *Ttys = NULL;
 struct Statmap_t Stats;
@@ -46,10 +52,13 @@ struct GOptmap_t GOpt = {
     .infod_port = "/var/run/.rplinfod_socket",
 };
 
-// LG vars
-static const char *m_idx = "pqrstuvwxyzabcdef", *s_idx = "0123456789abcdef";
+// Variables
+static unsigned char kver = 0;
+static pthread_mutex_t kver_lock = PTHREAD_MUTEX_INITIALIZER;
 
 //-----------------------------------------------------------------------------
+static const char *m_idx = "pqrstuvwxyzabcdef", *s_idx = "0123456789abcdef";
+
 char *G_devname_nm(uint32_t dev, char *buf, size_t count) {
     /* This function returns a "simple name" which can be integrated into
     rpld's log file name scheme. */
@@ -218,4 +227,26 @@ ssize_t send_wait(int fd, const void *buf, size_t count, int flags) {
     return count;
 }
 
+//-----------------------------------------------------------------------------
+inline static int K_MINORBITS(void) {
+    pthread_mutex_lock(&kver_lock);
+    if(kver == 0) { setup_kminorbits(); }
+    pthread_mutex_unlock(&kver_lock);
+    return (kver != 24) ? K26_MINORBITS : K24_MINORBITS;
+}
+
+static void setup_kminorbits(void) {
+    char buf[64];
+    FILE *fp;
+
+    memset(&buf, '\0', 64);
+    if((fp = fopen("/proc/version", "r")) == NULL) { return; }
+
+    fscanf(fp, "%*s %*s %60s", buf);
+    if(strncmp(buf, "2.6", 3) == 0) { kver = 26; }
+    else if(strncmp(buf, "2.4", 3) == 0) { kver = 24; }
+    fclose(fp);
+    return;
+}
+
 //==[ End of file ]============================================================
