@@ -37,6 +37,7 @@ user/replay.c - Realtime TTY log analyzer
 #include "dev.h"
 #include "rpl_packet.h"
 #include "global.h"
+#include "pctrl.h"
 
 #define offsetof(type, member) ((size_t)&((type *)NULL)->member)
 #define containerof(var, type, member) ((void *)(var) - offsetof(type, member))
@@ -44,6 +45,7 @@ user/replay.c - Realtime TTY log analyzer
 #define NANOSECOND  1000000000
 #define BUFSIZE     4096
 
+// Structure definitions
 enum { // current follow mode state:
     FM_NONE = 0,
     FM_CATCHUP, // playing old content
@@ -51,28 +53,25 @@ enum { // current follow mode state:
     FM_NODELAY, // same as FM_LIVE, but different code parts are executed
 };
 
-enum { // skip types
-    SKIP_NONE = 0,
-    SKIP_WRPACKET,
-    SKIP_TIME,
-};
-
 struct {
     double factor;
-    int follow, nodelay, seek_type;
-    long ovcorr, maxdelay, seek_cval;
+    int follow, nodelay, sktype, no_pctrl, showtime;
+    long ovcorr, maxdelay, skval;
 } Opt = {
-    .factor    = 1.0,       // -S
-    .follow    = 0,         // -F/-f
-    .maxdelay  = -1,        // -m
-    .nodelay   = 0,         // -T
-    .seek_type = SKIP_NONE, // -J/-j
-    .seek_cval = 0,
+    .factor   = 1.0,        // -S
+    .follow   = 0,          // -F/-f
+    .maxdelay = -1,         // -m
+    .nodelay  = 0,          // -T
+    .sktype   = PCTRL_NONE, // -J/-j
+    .skval    = 0,
+    .showtime = 0,          // -t
+    .no_pctrl = 0,          // --no-pctrl
 };
 
-static void replay_file(int);
+// Functions
+static int replay_file(int, const char *);
 
-static unsigned long calc_ovcorr(unsigned long, int) __attribute__((unused));
+static unsigned long calc_ovcorr(unsigned long, int);
 static int find_next_packet(int);
 static int get_options(int *, const char ***);
 inline static ssize_t read_nullfm(int, size_t);
@@ -86,8 +85,11 @@ inline static unsigned long long tv2usec(const struct timeval *);
 inline static void usec2tv(unsigned long long, struct timeval *);
 
 //-----------------------------------------------------------------------------
-int main(int argc, char **argv) {
-    if(!get_options(&argc, (const char ***)&argv)) { return EXIT_FAILURE; }
+int main(int argc, const char **argv) {
+    const char *auto_stdin[] = {"/dev/stdin", NULL};
+    int i;
+
+    if(!get_options(&argc, &argv)) { return EXIT_FAILURE; }
 
     fprintf(stderr,
       "> ttyreplay " TTYRPLD_VERSION "\n"
@@ -97,32 +99,69 @@ int main(int argc, char **argv) {
       "this program.\n\n"
     );
 
+    if(argc == 0) {
+        if(!isatty(STDIN_FILENO)) {
+            fprintf(stderr, "No arguments given and STDIN is not a tty --"
+             " replaying from STDIN\n");
+            argv = auto_stdin;
+        } else {
+            fprintf(stderr, "No filename(s) given.\n");
+            exit(EXIT_FAILURE);
+        }
+    }
+
     Opt.ovcorr = calc_ovcorr(0, 100);
 
-    while(*argv != NULL) {
-        int fd;
-        if((fd = open(*argv, O_RDONLY)) < 0) {
-            fprintf(stderr, "Could not open %s: %s\n", *argv++,
+    if(!Opt.no_pctrl) { pctrl_init(); }
+
+    for(i = 0; i < argc; ) {
+        int fd, rv;
+        if((fd = open(argv[i], O_RDONLY)) < 0) {
+            fprintf(stderr, "Could not open %s: %s\n", argv[i],
              strerror(errno));
+            ++i;
             continue;
         }
 
-        replay_file(fd);
+        rv = replay_file(fd, argv[i]);
         close(fd);
-        ++argv;
+
+        switch(rv) {
+            case PCTRL_PREV:
+                if(i > 0) { --i; }
+                break;
+            case PCTRL_NEXT:
+                if(i < argc - 1) { ++i; }
+                break;
+            case PCTRL_EXIT:
+                i = argc;
+                break;
+            default:
+                ++i;
+                break;
+        }
     }
 
+    pctrl_exit();
     return EXIT_SUCCESS;
 }
 
-static void replay_file(int fd) {
+static int replay_file(int fd, const char *name) {
     struct disk_packet packet;
+    struct pctrl_info ps;
     struct timeval stamp;
     ssize_t eax;
     long skew = 0;
     int tick = 0;
 
-    if(Opt.follow == FM_LIVE && !seek_to_end(fd)) { return; }
+    if(Opt.follow == FM_LIVE && !seek_to_end(fd)) { return 0; }
+
+
+    memset(&ps, 0, sizeof(struct pctrl_info));
+    ps.factor = Opt.factor;
+    ps.sktype = Opt.sktype;
+    ps.skval  = Opt.skval;
+    pctrl_activate(&ps);
 
     while((eax = read_waitfm(fd, &packet, sizeof(struct disk_packet))) ==
      sizeof(struct disk_packet)) {
@@ -144,14 +183,14 @@ static void replay_file(int fd) {
 
         switch(packet.event) {
             case EV_WRITE:
-                if(Opt.seek_type == SKIP_WRPACKET && Opt.seek_cval-- > 0) {
+                if(ps.sktype == PCTRL_SKPACK && ps.skval-- > 0) {
                     read_nullfm(fd, packet.size);
                     break;
                 }
                 if(!tick) {
                     // No delay after the first packet has been read...
                     ++tick;
-                    if(Opt.seek_cval > 0) {
+                    if(ps.skval > 0) {
                         // Urgh, these two lines are replicated below so much
                         memcpy(&stamp, &packet.tv, sizeof(struct timeval));
                         read_nullfm(fd, packet.size);
@@ -165,16 +204,17 @@ static void replay_file(int fd) {
                     creates a skew because of the usleep_ovcorr()! */
                     unsigned long long x;
                     tv_delta(&stamp, &packet.tv, &delta);
-                    if(Opt.seek_type == SKIP_TIME) {
-                        if((Opt.seek_cval -= tv2usec(&delta) / 1000) > 0) {
+                    if(ps.sktype == PCTRL_SKTIME) {
+                        if((ps.skval -= tv2usec(&delta) / 1000) > 0) {
                             memcpy(&stamp, &packet.tv, sizeof(struct timeval));
-                            read_nullfm(fd, packet.size);
+                            //read_nullfm(fd, packet.size);
+                            read_through(fd, STDOUT_FILENO, packet.size);
                             break;
                         }
                     }
-                    if(Opt.factor != 1.0) {
+                    if(ps.factor != 1.0) {
                         // -S switch for the fast
-                        usec2tv(tv2usec(&delta) / Opt.factor, &delta);
+                        usec2tv(tv2usec(&delta) / ps.factor, &delta);
                     }
                     if(Opt.maxdelay > 0 &&
                      (x = tv2usec(&delta)) > Opt.maxdelay) {
@@ -184,8 +224,26 @@ static void replay_file(int fd) {
                     }
                     usleep_ovcorr(&delta, &skew);
                 }
+
+                while(ps.paused) { sched_yield(); }
+
                 memcpy(&stamp, &packet.tv, sizeof(struct timeval));
                 read_through(fd, STDOUT_FILENO, packet.size);
+
+                if(Opt.showtime) {
+                    struct tm tm;
+                    char buf[80];
+                    time_t now = packet.tv.tv_sec;
+                    localtime_r(&now, &tm);
+                    if(Opt.showtime == 1) {
+                        strftime(buf, sizeof(buf), "\e7\e[1;300H\e[9D"
+                         "\e[0;7m[\e[0;1;37;41m%H:%M:%S\e[0;7m]\e8", &tm);
+                    } else {
+                        strftime(buf, sizeof(buf), "\e7\e[1;300H\e[12D"
+                         "\e[0;7m[\e[0;1;37;41m%d/%m %H:%M\e[0;7m]\e8", &tm);
+                    }
+                    write(STDOUT_FILENO, buf, strlen(buf));
+                }
                 break;
             case EV_IDENT:
                 fprintf(stderr, "\n" "Created using ");
@@ -199,14 +257,18 @@ static void replay_file(int fd) {
                 read_nullfm(fd, packet.size);
                 break;
         }
+
+        if(ps.brk) { break; }
     }
 
+    pctrl_deactivate(0);
+
     if(eax < 0) {
         perror("\n" "<Error while reading from stream>\n" "read():");
     }
 
-    printf("\n" "<Log replaying finished>\n");
-    return;
+    printf("\n" "<Log replaying of %s finished>\n", name);
+    return ps.brk;
 }
 
 //-----------------------------------------------------------------------------
@@ -337,6 +399,8 @@ static int get_options(int *argc, const char ***argv) {
     static const char *_empty_argv[] = {NULL, NULL};
     char *tmp = NULL;
     struct poptOption options_table[] = {
+        {"no-pctrl", 0, POPT_ARG_NONE, &Opt.no_pctrl, 0,
+         "Do not enable playing controls", NULL},
         {NULL, 'F', POPT_ARG_NONE, NULL, 'F',
          "Live feed follow mode (like `tail -f`)", NULL},
         {NULL, 'J', POPT_ARG_STRING, &tmp, 'J',
@@ -348,10 +412,13 @@ static int get_options(int *argc, const char ***argv) {
          "No delays (useful for text file grepping)", NULL},
         {NULL, 'f', POPT_ARG_NONE, NULL, 'f',
          "Catch-up follow mode (play file, switch to live feed on EOF)", NULL},
-        {NULL, 'j', POPT_ARG_STRING, &tmp, 'j',
+        {NULL, 'j', POPT_ARG_LONG, &Opt.skval, 'j',
          "Skip the given number of EV_WRITE packets", NULL},
         {NULL, 'm', POPT_ARG_LONG, &Opt.maxdelay, 'm',
          "Maximum delay to wait between packets", "msec"},
+        {NULL, 't', POPT_ARG_NONE, NULL, 't',
+         "Show the timestamp at the bottom of the screen"
+         " (May not work with all terminals!)", NULL},
         POPT_AUTOHELP
         POPT_TABLEEND
     };
@@ -366,11 +433,11 @@ static int get_options(int *argc, const char ***argv) {
             case 'F':
                 Opt.follow = FM_LIVE;
                 break;
-            case 'J': {
+            case 'J': { // jump to time
                 char *sec = "", *min = "", *hr = "";
 
-                Opt.seek_type = SKIP_TIME;
-                Opt.seek_cval = 0;
+                Opt.sktype = PCTRL_SKTIME;
+                Opt.skval  = 0;
 
                 if((sec = strrchr(tmp, ':')) == NULL) {
                     sec = tmp;
@@ -383,9 +450,8 @@ static int get_options(int *argc, const char ***argv) {
                         hr = tmp;
                     }
                 }
-                Opt.seek_cval = strtol(sec, NULL, 0) +
-                 60 * strtol(min, NULL, 0) + 3600 * strtol(hr, NULL, 0);
-                Opt.seek_cval *= 1000;
+                Opt.skval = (strtol(sec, NULL, 0) + 60 * strtol(min, NULL, 0) +
+                 3600 * strtol(hr, NULL, 0)) * 1000;
                 free(tmp);
                 break;
             }
@@ -395,14 +461,15 @@ static int get_options(int *argc, const char ***argv) {
             case 'f':
                 Opt.follow = FM_CATCHUP;
                 break;
-            case 'j':
-                Opt.seek_type = SKIP_WRPACKET;
-                Opt.seek_cval = strtol(tmp, NULL, 0);
-                free(tmp);
+            case 'j': // jump to packet
+                Opt.sktype = PCTRL_SKPACK;
                 break;
-            case 'm':
+            case 'm': // maximum delay between two packets
                 Opt.maxdelay *= 1000;
                 break;
+            case 't': // Show timestamp
+                ++Opt.showtime;
+                break;
         }
     }
 
