@@ -82,7 +82,7 @@ static int value_cmp(void *, void *);
 // Global variables (begin with uppercase character)
 static void *Buffer = NULL;
 static struct {
-    int _running, nocreat;
+    int _running, dolog;
     char *ovdev, *fbdev, *ofmt;
     long bsize;
 
@@ -92,7 +92,7 @@ static struct {
     .bsize       = 16384,
     .ovdev       = NULL,          // -D
     .fbdev       = "/dev/rpl:/dev/misc/rpl",
-    .nocreat     = 0,             // -Q
+    .dolog       = 1,             // !-Q
     .ofmt        = "%u@%d.%t.%l", // -O
     .infod_start = 0,             // -I
 };
@@ -104,8 +104,13 @@ int main(int argc, const char **argv) {
 
     /* Yep, the config file is what is needed by all three
     (rpld, infod, rplctl). */
-    if(!read_config_bp(*argv, "rpld.conf")) {
-        return EXIT_FAILURE;
+    if(!read_config("/etc/rpld.conf") && errno != ENOENT) {
+        fprintf(stderr, "/etc/rpld.conf exists but could not be read: %s\n",
+         strerror(errno));
+    }
+    if(!read_config_bp(*argv, "rpld.conf") && errno != ENOENT) {
+        fprintf(stderr, "$BINPATH/rpld.conf exists but could not be"
+         " read: %s\n", strerror(errno));
     }
 
     if(strcmp(basename(*argv), "rplctl") == 0) {
@@ -352,7 +357,7 @@ static int log_open(struct tty *tty) {
 
     // Open log
     HX_strclone(&tty->file, basename(buf));
-    if(Opt.nocreat) { snprintf(buf, MAXFNLEN, "/dev/null"); }
+    if(!Opt.dolog) { snprintf(buf, MAXFNLEN, "/dev/null"); }
     if((tty->fd = open(buf, O_WRONLY | O_CREAT | O_APPEND, 0600)) < 0) {
         notify(LOG_ERR, NF_NEWLINE, "Could not open %s: %s\n",
          buf, strerror(errno));
@@ -407,11 +412,17 @@ static int init_device(const char *in_devs) {
 
     if(Opt.ovdev == NULL && (new_dev = mknod_rpldev()) != NULL) {
         if((fd = open(new_dev, O_RDONLY)) == -1) {
+            int se = errno;
             new_dev = NULL;
-            perror("Could not connect to RPL device");
+            perror("dynamic_find: Could not connect to RPL device");
+            if(se == EBUSY) {
+                fprintf(stderr, "\t" "The RPL device can only be opened once,"
+                 "\n\t" "there is probably an instance of rpld running!\n");
+            }
         } else if(GOpt.verbose) {
             printf("Connected to RPL device\n");
         }
+        unlink(NODEPATH);
         unlink(new_dev);
     }
 
@@ -426,8 +437,13 @@ static int init_device(const char *in_devs) {
                 break;
             }
             if(errno != ENOENT) {
-                fprintf(stderr, "Could not open %s even though it exists: %s\n"
-                 "Trying next device...\n", devp, strerror(errno));
+                fprintf(stderr, "static_find: Could not open %s even though "
+                 "it exists: %s (trying next device)\n",
+                 devp, strerror(errno));
+            }
+            if(errno == EBUSY) {
+                fprintf(stderr, "\t" "The RPL device can only be opened once,"
+                 "\n\t" "there is probably an instance of rpld running!\n");
             }
             ++devp;
         }
@@ -535,9 +551,6 @@ static void sighandler_int(int s) {
     }
     if(GOpt.verbose) {
         printf("\n" "Received SIGINT/SIGTERM, shutting down.\n");
-    } else {
-        // If this is not done, process hangs until some fs activity
-        printf("\n");
     }
     Opt._running = 0;
     return;
@@ -569,8 +582,8 @@ static int get_options(int *argc, const char ***argv) {
          "Start INFOD subcomponent", NULL},
         {NULL, 'O', POPT_ARG_STRING, &Opt.ofmt, 0,
          "Override OFMT variable", "string"},
-        {NULL, 'Q', POPT_ARG_NONE, &Opt.nocreat, 0,
-         "Debug: do not create any files", NULL},
+        {NULL, 'Q', POPT_ARG_NONE, NULL, 'Q',
+         "Deactivate logging, only do bytecounts", NULL},
         {NULL, 'U', POPT_ARG_STRING, &tmp, 'U',
          "User to change to", "user"},
         {NULL, 'c', POPT_ARG_STRING, &tmp, 'c',
@@ -590,6 +603,9 @@ static int get_options(int *argc, const char ***argv) {
     ctx = poptGetContext(**argv, *argc, *argv, options_table, 0);
     while((c = poptGetNextOpt(ctx)) >= 0) {
         switch(c) {
+            case 'Q':
+                Opt.dolog = 0;
+                break;
             case 'U':
                 if(!pconfig_user(NULL, 0, tmp, NULL)) { return 0; }
                 free(tmp);
@@ -624,6 +640,7 @@ static int read_config(const char *file) {
         {"OVDEVICE", RCONF_STRING, &Opt.ovdev, NULL},
         {"FBDEVICE", RCONF_STRING, &Opt.fbdev, NULL},
         {"OFMT",     RCONF_STRING, &Opt.ofmt,  NULL},
+        {"DO_LOG",   RCONF_IBOOL,  &Opt.dolog, NULL},
         {"USER",     RCONF_CB,     NULL,       pconfig_user},
 
         {"INFOD_PORT", RCONF_STRING, &GOpt.infod_port, NULL},
